==> flutter_custom_form_component-0.0.5.tar.gz: lib/widget/field/date/mixin.dart <==
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/color.dart';
import 'package:flutter_custom_form_component/widget/field/date/formatter.dart';

/// Mixin supports 2 digits day or month, 2 digits month or day and 4 digits year text fields.
mixin DateFieldStateMixin {
  final TextEditingController _controller = TextEditingController();
  int? _day, _month, _year;
  final FocusNode _focusNode = FocusNode();
  late final bool _initialValid;
  late bool _isValid;
  late final int _maxLength;

  /// Call this on building the UI.
  Widget buildMixin(final BuildContext context,
          {required DateFieldFormatter formatter,
          required final String hint}) =>
      container(
          textField: TextField(
              controller: _controller,
              decoration: InputDecoration(
                  border: InputBorder.none, counterText: '', hintText: hint),
              focusNode: _focusNode,
              inputFormatters: [
                FilteringTextInputFormatter.digitsOnly,
                formatter
              ],
              keyboardType: Platform.isAndroid
                  ? TextInputType.datetime
                  : TextInputType.number,
              maxLength: _maxLength,
              style: _initialValid && _isValid
                  ? null
                  : const TextStyle(color: colorRed)));

  Widget container({required final TextField textField});

  /// Listener added to the controller.
  void _controllerListener() {
    onChanged();
    if (_controller.text.length == _maxLength) {
      _validate();
    }
  }

  /// Call this on diposing.
  void disposeMixin() {
    _controller.removeListener(_controllerListener);
    _controller.dispose();
    _focusNode.removeListener(_focusNodeListener);
    _focusNode.dispose();
  }

  /// Listener added to the focus node.
  void _focusNodeListener() {
    if (!_focusNode.hasFocus) {
      if (_controller.text.length < _maxLength) {
        _isValid = _controller.text.isEmpty;
        refresh();
      } else {
        _validate();
      }
    }
  }

  /// Call this on initializing.
  void initStateMixin(
      {required final bool initialValid, required final String separator}) {
    _initialValid = initialValid;
    _isValid = initialValid;

    /// 8 digits and 2 separators between.
    _maxLength = 8 + separator.length * 2;

    _controller.addListener(_controllerListener);
    _focusNode.addListener(_focusNodeListener);
  }

  /// Call this on changing text. _validate() should be outside of this implementation.
  void onChanged();

  /// Call this on refreshing the UI.
  void refresh();

  /// Split text to obtaining day, month and year.
  List<String> split({required final String separator}) =>
      _controller.text.split(separator);

  /// Validate the day, month and year.
  void _validate() {
    if (_year == null || _month == null || _day == null) {
      _isValid = false;
    } else {
      final DateTime date = DateTime(_year!, _month!, _day!);
      _isValid = date.year == _year && date.month == _month && date.day == _day;
    }
    refresh();
  }

  set day(final int? day) => _day = day;

  int? get day => _day;

  set month(final int? month) => _month = month;

  int? get month => _month;

  set year(final int? year) => _year = year;

  int? get year => _year;
}

==> flutter_custom_form_component-0.0.8.tar.gz: lib/widget/field/date/mixin.dart <==
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/color.dart';
import 'package:flutter_custom_form_component/widget/field/date/formatter.dart';

/// Mixin supports 2 digits day or month, 2 digits month or day and 4 digits year text fields.
mixin DateFieldStateMixin {
  final TextEditingController _controller = TextEditingController();
  final FocusNode _focusNode = FocusNode();
  late final bool _initialValid;
  late bool _isValid;
  late final int _maxLength;
  int? _year, _month, _day;

  /// Call this on building the UI.
  Widget buildMixin(final BuildContext context, {required final String hint}) =>
      container(
          textField:
              TextField(
                  controller: _controller,
                  decoration: InputDecoration(
                      border: InputBorder.none,
                      counterText: '',
                      hintText: hint),
                  focusNode: _focusNode,
                  inputFormatters: [
                    FilteringTextInputFormatter.digitsOnly,
                    formatter
                  ],
                  keyboardType: Platform.isAndroid
                      ? TextInputType.datetime
                      : TextInputType.number,
                  maxLength: _maxLength,
                  style: _initialValid && _isValid
                      ? null
                      : const TextStyle(color: colorRed)));

  Widget container({required final TextField textField});

  /// Listener added to the controller.
  void _controllerListener() {
    onChanged();
    if (_controller.text.length == _maxLength) {
      _validate();
    }
  }

  /// Call this on diposing.
  void disposeMixin() {
    _controller.removeListener(_controllerListener);
    _controller.dispose();
    _focusNode.removeListener(_focusNodeListener);
    _focusNode.dispose();
  }

  /// Listener added to the focus node.
  void _focusNodeListener() {
    if (!_focusNode.hasFocus) {
      if (_controller.text.length < _maxLength) {
        _isValid = _controller.text.isEmpty;
        refresh();
      } else {
        _validate();
      }
    }
  }

  /// Call this on initializing.
  void initStateMixin(
      {required final String? initialDate,
      required final bool initialValid,
      required final String separator}) {
    _initialValid = initialValid;
    _isValid = initialValid;

    /// 8 digits and 2 separators between.
    _maxLength = 8 + separator.length * 2;

    _controller.addListener(_controllerListener);
    _focusNode.addListener(_focusNodeListener);

    if (initialDate != null) {
      _controller.text = initialDate;
      _controller.selection =
          TextSelection.collapsed(offset: initialDate.length);
    }
  }

  /// Call this on changing text. _validate() should be outside of this implementation.
  void onChanged();

  /// Call this on refreshing the UI.
  void refresh();

  /// Split text to obtaining day, month and year.
  List<String> split({required final String separator}) =>
      _controller.text.split(separator);

  /// Validate the day, month and year.
  void _validate() {
    if (_year == null || _month == null || _day == null) {
      _isValid = false;
    } else {
      final DateTime date = DateTime(_year!, _month!, _day!);
      _isValid = date.year == _year && date.month == _month && date.day == _day;
    }
    refresh();
  }

  DateFieldFormatter get formatter;

  set year(final int? year) => _year = year;

  int? get year => _year;

  set month(final int? month) => _month = month;

  int? get month => _month;

  set day(final int? day) => _day = day;

  int? get day => _day;
}
